<?php
/**
 * @package   DocImport
 * @copyright Copyright (c)2011-2022 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\Component\DocImport\Administrator\CliCommand;

defined('_JEXEC') || die;

use Akeeba\Component\DocImport\Administrator\CliCommand\MixIt\ConfigureIO;
use Akeeba\Component\DocImport\Administrator\Model\CategoriesModel;
use Akeeba\Component\DocImport\Administrator\Model\XslModel;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Factory\MVCFactoryAwareTrait;
use Joomla\Console\Command\AbstractCommand;
use Joomla\Database\DatabaseAwareTrait;
use Joomla\Database\ParameterType;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

class Generate extends AbstractCommand
{
	use ConfigureIO;
	use DatabaseAwareTrait;
	use MVCFactoryAwareTrait;

	/**
	 * The default command name
	 *
	 * @since  4.0.5
	 * @var    string
	 */
	protected static $defaultName = 'docimport:generate';

	/**
	 * Configure the command.
	 *
	 * @return  void
	 *
	 * @since   7.5.0
	 */
	protected function configure(): void
	{
		$this->setHelp(Text::_('COM_DOCIMPORT_CLI_GENERATE_HELP'));
		$this->setDescription(Text::_('COM_DOCIMPORT_CLI_GENERATE_DESC'));

		$this->addArgument('category', InputArgument::OPTIONAL, Text::_('COM_DOCIMPORT_CLI_GENERATE_CATEGORY'));

		$this->addOption('force', 'f', InputOption::VALUE_NONE, Text::_('COM_DOCIMPORT_CLI_GENERATE_FORCE'));
	}

	/**
	 * @inheritDoc
	 */
	protected function doExecute(InputInterface $input, OutputInterface $output): int
	{
		// Configure the Symfony I/O
		$this->configureSymfonyIO($input, $output);

		// Disable database query logging (causes out–of–memory errors)
		$this->setDatabase(Factory::getContainer()->get('DatabaseDriver'));
		$this->getDatabase()->setMonitor(null);

		$catId      = $input->hasArgument('category') ? $input->getArgument('category') : null;
		$categories = !empty($catId) ? [(int) $catId] : $this->getCategories(!(bool) $input->getOption('force'));

		if (empty($categories))
		{
			$this->ioStyle->success(
				Text::_('COM_DOCIMPORT_CLI_GENERATE_LBL_NO_CAT_TO_UPDATE')
			);

			return 0;
		}

		/** @var XslModel $model */
		$model = $this->getMVCFactory()->createModel('Xsl', 'Administrator');

		foreach ($categories as $catId)
		{
			$title = $this->getTitle($catId);

			if ($title === null)
			{
				$this->ioStyle->error(
					Text::sprintf(
						'COM_DOCIMPORT_CLI_GENERATE_LBL_NO_SUCH_CAT',
						$catId
					)
				);
			}

			$this->ioStyle->info($title);

			try
			{
				$this->ioStyle->comment(Text::_('COM_DOCIMPORT_CLI_GENERATE_LBL_PROCESSING_XML'));
				$model->processXML($catId);
				$this->ioStyle->comment(Text::_('COM_DOCIMPORT_CLI_GENERATE_LBL_PROCESSING_ARTICLES'));
				$model->processFiles($catId);
			}
			catch (\Exception $e)
			{
				$this->ioStyle->error([
					$e->getMessage(),
					sprintf('%s(%u)', $e->getFile(), $e->getLine()),
					$e->getTraceAsString(),
				]);

				return 1;
			}
		}

		$this->ioStyle->success(
			Text::_('COM_DOCIMPORT_CLI_GENERATE_LBL_SUCCESS')
		);

		return 0;
	}

	private function getCategories(bool $onlyPublished = true): array
	{
		/** @var CategoriesModel $model */
		$model = $this->getMVCFactory()->createModel('Categories', 'Administrator', ['ignore_request' => true]);
		$model->getState();
		$model->setState('enabled', 1);

		$results = $model->getItems();

		$results = array_filter(
			$results,
			function (object $item) use ($onlyPublished) {
				if ($item->enabled != 1)
				{
					return false;
				}

				return $onlyPublished ? $item->status === 'modified' : true;
			}
		);

		return array_map(
			function (object $item) {
				return $item->docimport_category_id;
			}
			, $results
		);
	}

	private function getTitle(int $catId): ?string
	{
		$db    = $this->getDatabase();
		$query = $db->getQuery(true)
		            ->select($db->quoteName('title'))
		            ->from($db->quoteName('#__docimport_categories'))
		            ->where($db->quoteName('docimport_category_id') . ' = :catId')
		            ->bind(':catId', $catId, ParameterType::INTEGER);

		return $db->setQuery($query)->loadResult();
	}


}